;**********************************************************************
;    PROGRAMME DE TEST  DE LA CAPSULE POUR "AEIP" SUR R 5 GT TURBO    *
;**********************************************************************
;    NOM:      pres629a.asm                                           *
;    Date:     le Vendredi 07 mai 2004                                *
;    Revu:     le Mercredi 12 mai 2004                                *
;    Version:  1.4                                                    *
;    Circuit:  Allumage cartographique AEIP pour R 5 GT Turbo         *
;    Auteurs:  Philippe Quillon & Alain POLLET                        *
;    Edition et assemblage: MPLAB V5.70.00 and MPASM V2.70.00         *
;    Cadencement: 4Mhz Interne                                        *
;**********************************************************************      
;    Notes: Ce programme est adapt au PIC 12F629                     *
;           Attention au dpassement de la pile (stack) avec retlw 0  *
;           Donc, programmer avec deux niveaux de CALL maximum !      *
;           Include le fichier p12c508a.inc                           *
;           La valeur de temps de scrutation "T" dpend de l'cart de * 
;           temps qui spare deux impulsions capsule (frquence Osc). *
;           Temps de raction capsule pour dclenchement sortie      * 
;           500 ou 600 mbar (commutation courbe turbo du PIC16F628)   *
;           Soit: 1.50ms "court" (par dfaut) ou "long" de 1.60ms.    *
;           Si CONFIG1 ON = bit  1 = 1.60ms, sinon OFF = 1.50ms      * 
;           Le temps de raction capsule est fonction de l'tat       *
;           d'usure de la membrane, et du rglage de la frquence de  * 
;           sortie du circuit oscillateur (LM311) de ~ 200 Hz pour 0. *
;           Le signal est carr (5 V par div) sur la sortie AOP LM311.*
;           L'entre sur le PIC 12C508A fonctionne en rgime satur.  *       
;           Rglage pour frquence basse du circuit capsule (LM311)  *
;           ~ 100 Hz pour moins 300 mbar et 20 KHz pour 1 bar et plus.*        
;           Tempo pour "tat" des sorties 1,25s (pour 5 Volts).       *  
;           Le niveau de tension (4,81 Volts) sur OUT1/2 est li au   *  
;           temps de commutation et  la prsence d'une charge (led). * 
;           Avec 1,25 secondes, l'ondulation rsiduelle est de 1 Hz   *
;           sur les sorties, la tension est fixe (continue).          *
;           Ce niveau est le maximum que puisse fournir le PIC en     *
;           sorties TOR. Les transitions sur les sorties n'affectent  *
;           pas les charges commutes. Une entre TOR sur processeur  *
;           principal est donc possible, sans variation de niveau     *  
;           du bit de port correspondant.                             *  
;    Validation: Conversion du programme 16F84A pour PIC12C508A       * 
;                Sans flag d'IT (modif Alain POLLET)                  * 
;                Essais routiers concluants !!!                       *
;**********************************************************************
;* Versions du programme :                                            *
;*  V1.06 : Convertion du programme pour PIC 12F629 et simplification *
;*        : du programme                                              *
;*  V1.05 : Weak pull-ups (GP0,1,3) = 1 (disable). Sortie GP0/OUT3SCL,*
;*        : contrle du bit 6 TMR0,  1 si moins de 700 mbar + 8 STAT* 
;*  V1.04 : PSA  0 (/4), architecture 2 niveaux de CALL maximum !    *
;*        : Temps de raction 1.50ms "court" ou "long" de 1.60ms, OK !*
;*        : ATTENTION: Ne pas dpasser 1.70ms !!!                     *
;*  V1.03 : Nouveau programme, dbogage sur PIC 12C508A JW (09/05/04) *
;*        : PSA  0 (/4), 2 sorties TOR, OSCCAL activ, test b7 TMR0, *
;*        : dlai 1.25s pour sorties, instructions CALL rduites...   *
;*  V1.02 : Dip Switch, choix du temps de raction capsule            *
;*  V1.01 : Dip Switch, choix du temps de scrutation capsule          *
;*  V1.00 : Cration du programme, dbogage sur PIC 16F628 (03/05/04) *                                                                  
;**********************************************************************

        errorlevel -302               ;Suppression des messages "302" pour BANK 0 et 1

	list      p=12F629            ;Liste des directives pour dfinition du processeur 
	#include <p12F629.inc>        ;Dfinitions des variables spcifiques au processeur

	__CONFIG _INTRC_OSC_NOCLKOUT & _CP_OFF & _WDT_OFF & _MCLRE_OFF

;*********************************************************************
;                 PERIPHERIQUES ET CONTRLES                         *
;Connection PRES1 et PRES2 en GP4 et GP5:                            *
;Bit  bas  "b0" = OFF = 0 volts en sortie                           *
;Bit  haut "b1" = ON = 5 volts en sortie                            *
;                                                                    *
;Connection CONFIG1 en GP3:                                          *
;Bit  bas  "b0" = OFF = 0 volts en GP3                              *
;Bit  haut "b1" = ON = 5 volts en GP3                               *
;                                                                    *
;Connection SENSOR Capteur de pression (Circuit Capsule) en GP2      *
;Signal analogique niveau sur GP2/TOCK1 entre 0 et 2 bars            *
;                                                                    *
; GPIO:                                                              *
; SENSOR              Direct de LM311 et rsistance de               *
;                     polarisation 10K au 0 Volts                    *
; CFG1                +5 Volts en entre et rsistance de            *
;                     polarisation 10K au 0 Volts                    *
; OUT (1 & 2)         Pas de rsistance de polarisation au 0 Volts   *
;                     Sortie (+ 5 Volts)                             *
;*********************************************************************
;Brochage:
;                  _______  _______
;                 |       \/       |
;     (+5V) Vdd --+ 1            8 +-- Vss (GND)
;                 |                |
;   X1/CLKI/GP5 --+ 2            7 +-- GP0
;    (PRES 2)     |                |  (SCL IC, pour le futur) 
;   X2/CLKO/GP4 --+ 3            6 +-- GP1 
;    (PRES 1)     |                |  (SDA IC, pour le futur)
;     GP3/!MCLR --+ 4            5 +-- GP2
;    (Config 1)   |  (PIC12C508A)  |  (Pressure input)
;                 |________________|
;Graphe:
;
; Pression relative (en mbar) 
; 1024 |                   *
;      |                 *
; 800  |               *----- Seuil maxi !!!
;      |             *
; 600  |           *--------- Long  Seuil de dclenchement    
; 500  |         *----------- Court         
;      |       *
;      |     *
;      |   *
;      | *
;      |_________|__|__|_____ Temps de raction (en ms)
;      0         |  |  |
;              1.5 1.6 1.7
;
;***************************************************************
;*	Dclaration des Variables et Constantes globales       *
;*		de 0x20  0x5F (64 octets)                     *
;***************************************************************
	ORG	0x20
Pression	RES	1	; Variable contenant la valeur de bit0 en ATMO (0 ou 1)
;***************************************************************
;*	   Dclaration des CONSTANTES                          *
;*************************************************************** 
FALSE		EQU	0
TRUE		EQU	1
MSB		EQU	7
;***************************************************************
;*	   Dclaration des Macros pour le nom des E/S          *
;*************************************************************** 
#DEFINE		SCL	GPIO,0	; Port,0 est utilis comme entre IC - CLOCK
#DEFINE		SDA     GPIO,1	; Port,1 est utilis comme entre IC - DATA
#DEFINE		SENSOR	GPIO,2	; Port,2 est utilis comme entre capteur Pression
#DEFINE		CFG1	GPIO,3	; Port,3 est utilis comme entre configuration (DIP Switch)
#DEFINE		PRES1	GPIO,4	; Port,4 est utilis comme sortie TOR N1
#DEFINE		PRES2	GPIO,5	; Port,5 est utilis comme sortie TOR N2
;*********************************************************************
;               INITIALISATION DU PROGRAMME PRINCIPAL                *
;*********************************************************************
;*********************************************************************
	ORG     0x000           ; VECTEUR RESET  l'adresse 0 
Main_Config
	;--- Initialisation du Port GPIO
	;---------------------------------------------------------------------
	BSF	STATUS,RP0	; Bank1
        MOVLW   0x00            ; W <-- h'00'
        MOVWF   VRCON           ; Les rfrences de voltage sont inhibes
	BCF	STATUS,RP0	; Bank0
	CLRF	GPIO		; PORT GPIO pins direction setup ==> 1=input, 0=output
	MOVLW	h'07'		; 
	MOVWF	CMCON 		; SET GP<2:0> en entre/sortie
	BSF	STATUS,RP0	; Bank1
	MOVLW	b'00001111'	; Initialisation du port I/O
	MOVWF	TRISIO		; Configuration du port E/S
	;---------------------------------------------------------------------                            
	;--- OPTION setup       ; Initialisation du programme :
	;---------------------------------------------------------------------
	movlw b'11001001'	; Incrmentation "internal clock" TMR0 et PSA  0 (/4) !  ***
	     ;  x-------	; 7, 0=enable, 1=disable, wake-up on pin change (GP0,1,3) ==> NON 
	     ;  -x------	; 6, 0=enable, 1=disable, weak pull-ups (GP0,1,3)         ==> NON
	     ;  --x-----	; 5, timer0 source, 0=internal clock, 1=T0CKI             ==> NON
	     ;  ---x----	; 4, timer0 ext edge, 0=Low2high, 1=high2low (T0CK1 pin)  ==> Interne
	     ;  ----x---	; 3, prescaler assign, 1=wdt (sans le fuse), 0=timer0     ==> vers Timer0
	     ;  -----x--	; 2,1,0, timer0 prescaler rate select
	     ;  ------x-	; - 000=2, 001=4, 010=8, 011=16, 100=32,..., 111=256.     ==> PS:64
	     ;  -------x	; 
	MOVWF	OPTION_REG	; Chargement du registre OPTION   
	BCF	STATUS,RP0	; Bank0
Main_Init
	CLRF	Pression	; Pression <-- 0
	
;*********************************************************************
;                    DEBUT DU PROGRAMME PRINCIPAL                    *
;*********************************************************************
;*********************************************************************
Main
Main_wait0
	BTFSC	SENSOR		; On teste si un niveau bas capsule est prsent ?
	goto    Main_wait0	; Non, On attend...                                 
	NOP
	NOP

Main_front_1
	BTFSS	SENSOR		; On teste si un front montant capsule est prsent ?
	goto    Main_front_1	; Non, On attend...                                 
	CLRF    TMR0
	NOP
	NOP
	NOP
Main_front_0
	BTFSC	SENSOR		; On teste si un front descendant capsule est prsent ?
	goto    Main_front_0	; Non, On attend...                                 
	NOP
	NOP
	NOP
	NOP
Main_front_1_2
	BTFSS	SENSOR		; On teste si un front montant capsule est prsent ?
	goto    Main_front_1_2	; Non, On attend...                                 
	NOP
	NOP
	NOP
	NOP
Main_front_0_2
	BTFSC	SENSOR		; On teste si un front descendant capsule est prsent ?
	goto    Main_front_0_2	; Non, On attend...                                 
	NOP
	NOP
	NOP
	NOP
  
Main_fin_periode
	BTFSS	SENSOR		; On teste si un front montant capsule est prsent ?
	GOTO	Main_fin_periode; Non, On attend ...
                                ; Oui, temps de scrutation "T" termin  !
	MOVFW	TMR0		; Timer <-- TMR0
	MOVWF	Pression	;
        CLRF    TMR0 		; TMR0 <-- 0

	BTFSS	Pression,6	; PRES1 <- TIMER[Bit7]
	BCF	PRES1		;
	BTFSC	Pression,6	;
	BSF	PRES1		;
	BTFSS	Pression,5	; PRES2 <- TIMER[Bit6]
	BCF	PRES2		;
	BTFSC	Pression,5	;
	BSF	PRES2		;
        goto    Main 		; Boucler...

;********************************************************************
	org 0x2100		;Adresse dbut zone EEPROM
Version	DE	"1.0.6"	        ;Version du programme
;********************************************************************
	        END		        ; Directive fin de programme